@@ -1,6 +1,6 @@
 /* cfftot1.cc -- driver for translating CFF fonts to Type 1 fonts
  *
- * Copyright (c) 2002-2010 Eddie Kohler
+ * Copyright (c) 2002-2013 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -123,32 +123,39 @@ do_file(const char *infn, const char *outfn, PermString name, ErrorHandler *errh
 
     if (c == EOF)
 	errh->fatal("%s: empty file", infn);
-    if (c == 1 || c == 'O') {
-	StringAccum sa(150000);
-	while (!feof(f)) {
-	    int forward = fread(sa.reserve(32768), 1, 32768, f);
-	    sa.adjust_length(forward);
-	}
-	if (f != stdin)
-	    fclose(f);
-
-	LandmarkErrorHandler cerrh(errh, infn);
-	String data = sa.take_string();
-	if (c == 'O')
-	    data = Efont::OpenType::Font(data, &cerrh).table("CFF");
-
-	Cff *cff = new Cff(data, &cerrh);
-	Cff::FontParent *fp = cff->font(name, &cerrh);
-	if (errh->nerrors() == 0
-	    && !(font = dynamic_cast<Cff::Font *>(fp)))
-	    errh->fatal("%s: CID-keyed fonts not supported", infn);
-    } else
+    if (c != 1 && c != 'O')
 	errh->fatal("%s: not a CFF or OpenType/CFF font", infn);
 
+    StringAccum sa(150000);
+    int amt;
+    do {
+	if (char *x = sa.reserve(32768)) {
+	    amt = fread(x, 1, 32768, f);
+	    sa.adjust_length(amt);
+	} else
+	    amt = 0;
+    } while (amt != 0);
+    if (!feof(f) || ferror(f))
+	errh->lerror(infn, "%s", strerror(errno));
+    if (f != stdin)
+	fclose(f);
+
+    ContextErrorHandler cerrh(errh, "While processing %s:", infn);
+    cerrh.set_indent(0);
+    String data = sa.take_string();
+    if (c == 'O')
+	data = Efont::OpenType::Font(data, &cerrh).table("CFF");
+
+    Cff *cff = new Cff(data, &cerrh);
+    Cff::FontParent *fp = cff->font(name, &cerrh);
+    if (errh->nerrors() == 0
+	&& !(font = dynamic_cast<Cff::Font *>(fp)))
+	errh->fatal("%s: CID-keyed fonts not supported", infn);
+
     if (errh->nerrors() > 0)
 	return;
 
-    Type1Font *font1 = create_type1_font(font, errh);
+    Type1Font *font1 = create_type1_font(font, &cerrh);
 
     if (!outfn || strcmp(outfn, "-") == 0) {
 	f = stdout;
@@ -210,7 +217,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("cfftot1 (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2002-2010 Eddie Kohler\n\
+	    printf("Copyright (C) 2002-2013 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
