@@ -2,7 +2,7 @@
 /* clp.c - Complete source code for CLP.
  * This file is part of CLP, the command line parser package.
  *
- * Copyright (c) 1997-2010 Eddie Kohler, ekohler@gmail.com
+ * Copyright (c) 1997-2012 Eddie Kohler, ekohler@gmail.com
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -275,7 +275,7 @@ encode_utf8(char *s, int n, int c)
 }
 
 static int
-decode_utf8(const char *s, char **cp)
+decode_utf8(const char *s, const char **cp)
 {
     int c;
     if ((unsigned char) *s <= 0x7F)		/* 1 byte:  0x000000-0x00007F */
@@ -319,14 +319,14 @@ decode_utf8(const char *s, char **cp)
 	    /* nothing */;
     }
     if (cp)
-	*cp = (char *) s;
+	*cp = s;
     return c;
 }
 
 static int
 utf8_charlen(const char *s)
 {
-    char *sout;
+    const char *sout;
     (void) decode_utf8(s, &sout);
     return sout - s;
 }
@@ -364,7 +364,7 @@ long_as_short(const Clp_Internal *cli, const Clp_Option *o,
     if ((cli->long1pos || cli->long1neg) && io->ilong) {
 	const char *name = o->long_name + io->ilongoff;
 	if (cli->utf8) {
-	    int c = decode_utf8(name, (char **) &name);
+	    int c = decode_utf8(name, &name);
 	    if (!*name && c && c != U_REPLACEMENT)
 		return c;
 	} else if (name[0] && !name[1])
@@ -393,11 +393,11 @@ compare_options(Clp_Parser *clp, const Clp_Option *o1, Clp_InternOption *io1,
 	int short2 = (io2->ishort ? o2->short_name : -3);
 	int shortx2 = long_as_short(cli, o2, io2, -4);
 	if (short1 == short2)
-	    Clp_OptionError(clp, "CLP internal error: more than 1 option has short name %`%c%'", short1);
+	    Clp_OptionError(clp, "CLP internal error: more than 1 option has short name %<%c%>", short1);
 	else if ((short1 == shortx2 || shortx1 == short2 || shortx1 == shortx2)
 		 && ((io1->ipos && io2->ipos && cli->long1pos)
 		     || (io1->ineg && io2->ineg && cli->long1neg)))
-	    Clp_OptionError(clp, "CLP internal error: 1-char long name conflicts with short name %`%c%'", (short1 == shortx2 ? shortx2 : shortx1));
+	    Clp_OptionError(clp, "CLP internal error: 1-char long name conflicts with short name %<%c%>", (short1 == shortx2 ? shortx2 : shortx1));
     }
 
     /* analyze longest minimum match */
@@ -419,7 +419,7 @@ compare_options(Clp_Parser *clp, const Clp_Option *o1, Clp_InternOption *io1,
 	if (io2->ilong) {
 	    const char *name2 = o2->long_name + io2->ilongoff;
 	    if (strcmp(name1, name2) == 0)
-		Clp_OptionError(clp, "CLP internal error: duplicate long name %`%s%'", name1);
+		Clp_OptionError(clp, "CLP internal error: duplicate long name %<%s%>", name1);
 	    if (io1->ipos && io2->ipos && !strncmp(name1, name2, io1->lmmpos)
 		&& (!io1->iprefmatch || strncmp(name1, name2, strlen(name1))))
 		io1->lmmpos = min_different_chars(name1, name2);
@@ -448,14 +448,14 @@ compare_options(Clp_Parser *clp, const Clp_Option *o1, Clp_InternOption *io1,
  * first argument returned by Clp_Next() will be the second argument in @a
  * argv.  Note that this behavior differs from Clp_SetArguments.</li>
  * <li>UTF-8 support is on iff the <tt>LANG</tt> environment variable contains
- * one of the substrings "UTF-8", "UTF8", or "utf8".</li>
+ * one of the substrings "UTF-8", "UTF8", or "utf8".  Override this with
+ * Clp_SetUTF8().</li>
  * <li>The Clp_ValString, Clp_ValStringNotOption, Clp_ValInt, Clp_ValUnsigned,
  * Clp_ValBool, and Clp_ValDouble types are installed.</li>
  * <li>Errors are reported to standard error.</li>
  * </ul>
  *
- * To override these characteristics, either call functions like
- * Clp_SetUTF8(), or create your Clp_Parser with no arguments or options
+ * You may also create a Clp_Parser with no arguments or options
  * (<tt>Clp_NewParser(0, 0, 0, 0)</tt>) and set the arguments and options
  * later.
  *
@@ -471,9 +471,12 @@ Clp_NewParser(int argc, const char * const *argv, int nopt, const Clp_Option *op
     Clp_Parser *clp = (Clp_Parser *)malloc(sizeof(Clp_Parser));
     Clp_Internal *cli = (Clp_Internal *)malloc(sizeof(Clp_Internal));
     Clp_InternOption *iopt = (Clp_InternOption *)malloc(sizeof(Clp_InternOption) * nopt);
-    if (!clp || !cli || !iopt)
+    if (cli)
+	cli->valtype = (Clp_ValType *)malloc(sizeof(Clp_ValType) * Clp_InitialValType);
+    if (!clp || !cli || !iopt || !cli->valtype)
 	goto failed;
 
+    clp->opt = -1;
     clp->negated = 0;
     clp->have_val = 0;
     clp->vstr = 0;
@@ -508,11 +511,7 @@ Clp_NewParser(int argc, const char * const *argv, int nopt, const Clp_Option *op
     cli->long1pos = cli->long1neg = 0;
 
     /* Add default type parsers */
-    cli->valtype = (Clp_ValType *)malloc(sizeof(Clp_ValType) * Clp_InitialValType);
-    if (!cli->valtype)
-	goto failed;
     cli->nvaltype = 0;
-
     Clp_AddType(clp, Clp_ValString, 0, parse_string, 0);
     Clp_AddType(clp, Clp_ValStringNotOption, Clp_DisallowOptions, parse_string, 0);
     Clp_AddType(clp, Clp_ValInt, 0, parse_int, 0);
@@ -612,6 +611,13 @@ Clp_SetUTF8(Clp_Parser *clp, int utf8)
     return old_utf8;
 }
 
+/** @param clp the parser
+ * @param c character
+ * @return option character treatment
+ *
+ * Returns an integer specifying how CLP treats arguments that begin
+ * with character @a c.  See Clp_SetOptionChar for possibilities.
+ */
 int
 Clp_OptionChar(Clp_Parser *clp, int c)
 {
@@ -1068,28 +1074,28 @@ parse_string(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 static int
 parse_int(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 {
-    char *val;
+    const char *val;
     if (*arg == 0 || isspace((unsigned char) *arg)
 	|| (user_data != 0 && *arg == '-'))
-	val = (char *) arg;
+	val = arg;
     else if (user_data != 0) {	/* unsigned */
 #if HAVE_STRTOUL
-	clp->val.u = strtoul(arg, &val, 0);
+	clp->val.u = strtoul(arg, (char **) &val, 0);
 #else
 	/* don't bother really trying to do it right */
 	if (arg[0] == '-')
-	    val = (char *) arg;
+	    val = arg;
 	else
-	    clp->val.u = strtol(arg, &val, 0);
+	    clp->val.u = strtol(arg, (char **) &val, 0);
 #endif
     } else
-	clp->val.i = strtol(arg, &val, 0);
+	clp->val.i = strtol(arg, (char **) &val, 0);
     if (*arg != 0 && *val == 0)
 	return 1;
     else if (complain) {
 	const char *message = user_data != 0
-	    ? "%`%O%' expects a nonnegative integer, not %`%s%'"
-	    : "%`%O%' expects an integer, not %`%s%'";
+	    ? "%<%O%> expects a nonnegative integer, not %<%s%>"
+	    : "%<%O%> expects an integer, not %<%s%>";
 	return Clp_OptionError(clp, message, arg);
     } else
 	return 0;
@@ -1098,16 +1104,16 @@ parse_int(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 static int
 parse_double(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 {
-    char *val;
+    const char *val;
     (void)user_data;
     if (*arg == 0 || isspace((unsigned char) *arg))
-	val = (char *) arg;
+	val = arg;
     else
-	clp->val.d = strtod(arg, &val);
+	clp->val.d = strtod(arg, (char **) &val);
     if (*arg != 0 && *val == 0)
 	return 1;
     else if (complain)
-	return Clp_OptionError(clp, "%`%O%' expects a real number, not %`%s%'", arg);
+	return Clp_OptionError(clp, "%<%O%> expects a real number, not %<%s%>", arg);
     else
 	return 0;
 }
@@ -1139,7 +1145,7 @@ parse_bool(Clp_Parser *clp, const char *arg, int complain, void *user_data)
 
   error:
     if (complain)
-	Clp_OptionError(clp, "%`%O%' expects a true-or-false value, not %`%s%'", arg);
+	Clp_OptionError(clp, "%<%O%> expects a true-or-false value, not %<%s%>", arg);
     return 0;
 }
 
@@ -1170,21 +1176,24 @@ parse_string_list(Clp_Parser *clp, const char *arg, int complain, void *user_dat
     }
 
     if (complain) {
-	const char *complaint = (ambiguous ? "ambiguous" : "invalid");
-	if (!ambiguous) {
+	const char *complaint;
+	if (ambiguous)
+	    complaint = "ambiguous value %<%s%> for option %<%O%>";
+	else {
+	    complaint = "unknown value %<%s%> for option %<%O%>";
 	    ambiguous = sl->nitems_invalid_report;
 	    for (idx = 0; idx < ambiguous; idx++)
 		ambiguous_values[idx] = idx;
 	}
 	return ambiguity_error
 	    (clp, ambiguous, ambiguous_values, sl->items, sl->iopt,
-	     "", "option %`%O%' value %`%s%' is %s", arg, complaint);
+	     "", complaint, arg);
     } else
 	return 0;
 }
 
 
-int
+static int
 finish_string_list(Clp_Parser *clp, int val_type, int flags,
 		   Clp_Option *items, int nitems, int itemscap)
 {
@@ -1516,7 +1525,7 @@ get_oclass(Clp_Parser *clp, const char *text, int *ocharskip)
 {
     int c;
     if (clp->internal->utf8) {
-	char *s;
+	const char *s;
 	c = decode_utf8(text, &s);
 	*ocharskip = s - text;
     } else {
@@ -1815,7 +1824,8 @@ Clp_Next(Clp_Parser *clp)
     /* Get the next argument or option */
     if (!next_argument(clp, cli->option_processing ? 0 : 2)) {
 	clp->val.s = clp->vstr;
-	return clp->have_val ? Clp_NotOption : Clp_Done;
+	clp->opt = clp->have_val ? Clp_NotOption : Clp_Done;
+	return clp->opt;
     }
 
     clp->negated = cli->whole_negated;
@@ -1844,16 +1854,16 @@ Clp_Next(Clp_Parser *clp)
 	if (cli->ambiguous)
 	    ambiguity_error(clp, cli->ambiguous, cli->ambiguous_values,
 			    cli->opt, cli->iopt, cli->option_chars,
-			    "option %`%s%s%' is ambiguous",
+			    "option %<%s%s%> is ambiguous",
 			    cli->option_chars, cli->xtext);
 	else if (cli->is_short && !cli->could_be_short)
-	    Clp_OptionError(clp, "unrecognized option %`%s%C%'",
+	    Clp_OptionError(clp, "unrecognized option %<%s%C%>",
 			    cli->option_chars, cli->xtext);
 	else
-	    Clp_OptionError(clp, "unrecognized option %`%s%s%'",
+	    Clp_OptionError(clp, "unrecognized option %<%s%s%>",
 			    cli->option_chars, cli->xtext);
 
-	return Clp_BadOption;
+	return (clp->opt = Clp_BadOption);
     }
 
     /* Set the current option */
@@ -1865,20 +1875,21 @@ Clp_Next(Clp_Parser *clp)
     if (clp->negated
 	|| (!cli->iopt[optno].imandatory && !cli->iopt[optno].ioptional)) {
 	if (clp->have_val) {
-	    Clp_OptionError(clp, "%`%O%' can't take an argument");
-	    return Clp_BadOption;
+	    Clp_OptionError(clp, "%<%O%> can%,t take an argument");
+	    clp->opt = Clp_BadOption;
 	} else
-	    return cli->opt[optno].option_id;
+	    clp->opt = cli->opt[optno].option_id;
+	return clp->opt;
     }
 
     /* Get an argument if we need one, or if it's optional */
     /* Sanity-check the argument type. */
     opt = &cli->opt[optno];
     if (opt->val_type <= 0)
-	return Clp_Error;
+	return (clp->opt = Clp_Error);
     vtpos = val_type_binsearch(cli, opt->val_type);
     if (vtpos == cli->nvaltype || cli->valtype[vtpos].val_type != opt->val_type)
-	return Clp_Error;
+	return (clp->opt = Clp_Error);
 
     /* complain == 1 only if the argument was explicitly given,
        or it is mandatory. */
@@ -1895,17 +1906,19 @@ Clp_Next(Clp_Parser *clp)
 	    int got_option = cli->xtext != 0;
 	    Clp_RestoreParser(clp, &clpsave);
 	    if (got_option)
-		Clp_OptionError(clp, "%`%O%' requires a non-option argument");
+		Clp_OptionError(clp, "%<%O%> requires a non-option argument");
 	    else
-		Clp_OptionError(clp, "%`%O%' requires an argument");
-	    return Clp_BadOption;
+		Clp_OptionError(clp, "%<%O%> requires an argument");
+	    return (clp->opt = Clp_BadOption);
 	}
 
     } else if (cli->is_short && !clp->have_val
-	       && cli->xtext[clp_utf8_charlen(cli, cli->xtext)])
+	       && cli->xtext[clp_utf8_charlen(cli, cli->xtext)]) {
 	/* The -[option]argument case:
 	   Assume that the rest of the current string is the argument. */
 	next_argument(clp, 1);
+	complain = 1;
+    }
 
     /* Parse the argument */
     if (clp->have_val) {
@@ -1913,14 +1926,14 @@ Clp_Next(Clp_Parser *clp)
 	if (atr->func(clp, clp->vstr, complain, atr->user_data) <= 0) {
 	    /* parser failed */
 	    clp->have_val = 0;
-	    if (cli->iopt[optno].imandatory)
-		return Clp_BadOption;
+	    if (complain)
+		return (clp->opt = Clp_BadOption);
 	    else
 		Clp_RestoreParser(clp, &clpsave);
 	}
     }
 
-    return opt->option_id;
+    return (clp->opt = opt->option_id);
 }
 
 
@@ -2116,11 +2129,14 @@ Clp_VaOptionError(Clp_Parser *clp, Clp_BuildString *bs,
 		*bs->pos++ = '%';
 	    break;
 
-	  case '`':
+	  case '`':		/* backwards compatibility */
+	  case '<':
 	    append_build_string(bs, (cli->utf8 ? "\342\200\230" : "'"), -1);
 	    break;
 
-	  case '\'':
+	  case '\'':		/* backwards compatibility */
+	  case ',':
+	  case '>':
 	    append_build_string(bs, (cli->utf8 ? "\342\200\231" : "'"), -1);
 	    break;
 
@@ -2185,12 +2201,15 @@ do_error(Clp_Parser *clp, Clp_BuildString *bs)
  * current option is defined in the Clp_Parser object itself.</dd>
  * <dt><tt>%%</tt></dt>
  * <dd>Prints a percent character.</dd>
- * <dt><tt>%</tt><tt>`</tt></dt>
- * <dd>In UTF-8 mode, prints a left single quote.  Otherwise prints a single
- * quote.</dd>
- * <dt><tt>%</tt><tt>'</tt></dt>
- * <dd>In UTF-8 mode, prints a right single quote.  Otherwise prints a single
- * quote.</dd>
+ * <dt><tt>%</tt><tt>&lt;</tt></dt>
+ * <dd>Prints an open quote string.  In UTF-8 mode, prints a left single
+ * quote.  Otherwise prints a single quote.</dd>
+ * <dt><tt>%</tt><tt>&gt;</tt></dt>
+ * <dd>Prints a closing quote string.  In UTF-8 mode, prints a right single
+ * quote.  Otherwise prints a single quote.</dd>
+ * <dt><tt>%</tt><tt>,</tt></dt>
+ * <dd>Prints an apostrophe.  In UTF-8 mode, prints a right single quote.
+ * Otherwise prints a single quote.</dd>
  * </dl>
  *
  * Note that no flag characters, precision, or field width characters are
