@@ -1,6 +1,6 @@
 /* otfinfo.cc -- driver for reporting information about OpenType fonts
  *
- * Copyright (c) 2003-2011 Eddie Kohler
+ * Copyright (c) 2003-2013 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -136,7 +136,7 @@ String
 read_file(String filename, ErrorHandler *errh, bool warning = false)
 {
     FILE *f;
-    int f_errno;
+    int f_errno = 0;
     if (!filename || filename == "-") {
 	filename = "<stdin>";
 	f = stdin;
@@ -558,7 +558,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("otfinfo (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2003-2011 Eddie Kohler\n\
+	    printf("Copyright (C) 2003-2013 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
