@@ -1,6 +1,6 @@
 /* automatic.{cc,hh} -- code for automatic mode and interfacing with kpathsea
  *
- * Copyright (c) 2003-2010 Eddie Kohler
+ * Copyright (c) 2003-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -40,6 +40,20 @@
 
 #ifdef WIN32
 # define mkdir(dir, access) mkdir(dir)
+# define COPY_CMD "copy"
+# define CMD_SEP "&"
+#else
+# define COPY_CMD "cp"
+# define CMD_SEP ";"
+#endif
+
+/* kpathsea may already have defined this */
+#ifndef DEV_NULL
+# ifdef WIN32
+#  define DEV_NULL "NUL"
+# else
+#  define DEV_NULL "/dev/null"
+# endif
 #endif
 
 #if HAVE_AUTO_T1DOTLESSJ
@@ -482,7 +496,7 @@ installed_truetype(const String &ttf_filename, bool allow_generate, ErrorHandler
 
 	int retval;
 	if (!same_filename(ttf_filename, installed_ttf_filename)) {
-	    String command = "cp " + shell_quote(ttf_filename) + " " + shell_quote(installed_ttf_filename);
+	    String command = COPY_CMD " " + shell_quote(ttf_filename) + " " + shell_quote(installed_ttf_filename);
 	    retval = mysystem(command.c_str(), errh);
 	    if (retval == 127)
 		errh->error("could not run %<%s%>", command.c_str());
@@ -535,7 +549,7 @@ installed_type42(const String &ttf_filename, const String &ps_fontname, bool all
 #endif
 
 #if HAVE_AUTO_TTFTOTYPE42
-    // if not found, and can generate on the fly, run cfftot1
+    // if not found, and can generate on the fly, run ttftotype42
     if (allow_generate && ttf_filename && ttf_filename != "-" && getodir(O_TYPE42, errh)) {
 	String t42_filename = odir[O_TYPE42] + "/" + ps_fontname + ".t42";
 	if (t42_filename.find_left('\'') >= 0 || ttf_filename.find_left('\'') >= 0)
@@ -578,7 +592,7 @@ update_autofont_map(const String &fontname, String mapline, ErrorHandler *errh)
 
 	int fd = open(map_file.c_str(), O_RDWR | O_CREAT, 0666);
 	if (fd < 0)
-	    return errh->error("%s: %s", map_file.c_str(), strerror(errno));
+	    return errh->lerror(map_file, "%s", strerror(errno));
 	FILE *f = fdopen(fd, "r+");
 	// NB: also change encoding logic if you change this code
 
@@ -602,14 +616,16 @@ update_autofont_map(const String &fontname, String mapline, ErrorHandler *errh)
 
 	// read old data from map file
 	StringAccum sa;
-	while (!feof(f))
+	int amt;
+	do {
 	    if (char *x = sa.reserve(8192)) {
-		int amt = fread(x, 1, 8192, f);
+		amt = fread(x, 1, 8192, f);
 		sa.adjust_length(amt);
-	    } else {
-		fclose(f);
-		return errh->error("Out of memory!");
-	    }
+	    } else
+		amt = 0;
+	} while (amt != 0);
+	if (!feof(f))
+	    return errh->error("%s: %s", map_file.c_str(), strerror(errno));
 	String text = sa.take_string();
 
 	// add comment if necessary
@@ -725,11 +741,11 @@ update_autofont_map(const String &fontname, String mapline, ErrorHandler *errh)
 	    int slash = filename.find_right('\'');
 	    if (slash >= 0)
 		filename = filename.substring(slash + 1);
-	    String command = "updmap --nomkmap --enable Map " + shell_quote(filename) + "; updmap";
+	    String command = "updmap --nomkmap --enable Map " + shell_quote(filename) + CMD_SEP " updmap";
 	    if (verbose)
 		command += " 1>&2";
 	    else
-		command += " >/dev/null 2>&1";
+		command += " >" DEV_NULL " 2>&1";
 	    int retval = mysystem(command.c_str(), errh);
 	    if (retval == 127)
 		errh->warning("could not run %<%s%>", command.c_str());
