@@ -1,6 +1,6 @@
 /* util.{cc,hh} -- various bits
  *
- * Copyright (c) 2003-2011 Eddie Kohler
+ * Copyright (c) 2003-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -103,15 +103,34 @@ same_filename(const String &a, const String &b)
 String
 shell_quote(const String &str)
 {
-#if defined(_MSDOS) || defined(_WIN32)
-    return str;			// XXX
-#else
     if (!str)
-	return String::make_stable("''");
+	return String::make_stable("\"\"");
 
     const char *begin = str.begin();
     const char *end = str.end();
     StringAccum sa;
+
+#if defined(_MSDOS) || defined(_WIN32)
+    sa.append('\"');
+
+    for (const char *s = begin; s < end; ++s)
+	if (isalnum((unsigned char) *s) || *s == '_' || *s == '-' || *s == '+' || *s == '\\' || *s == ':' || *s == '.')
+	    /* do nothing */;
+	else if (*s == '\"') {
+	    sa.append(begin, s);
+	    sa.append("\"\"\"", 3);
+	    begin = s + 1;
+	} else {
+	    sa.append(begin, s + 1);
+	    begin = s + 1;
+	}
+
+    if (sa.length() > 1) {
+	sa.append(begin, end);
+	sa.append('\"');
+	return sa.take_string();
+    }
+#else
     for (const char *s = begin; s < end; s++)
 	if (isalnum((unsigned char) *s) || *s == '_' || *s == '-' || *s == '+' || *s == '/' || *s == ':' || *s == '.')
 	    /* do nothing */;
@@ -121,13 +140,13 @@ shell_quote(const String &str)
 	    begin = s;
 	}
 
-    if (!sa)
-	return str;
-    else {
+    if (sa.length()) {
 	sa.append(begin, end);
 	return sa.take_string();
     }
 #endif
+
+    return str;
 }
 
 int
@@ -143,13 +162,26 @@ mysystem(const char *command, ErrorHandler *errh)
     }
 }
 
+#if !HAVE_MKSTEMP
+static const char *
+my_tmpnam()
+{
+# ifndef WIN32
+    return tmpnam(0);
+# else
+    const char *dir = getenv("TEMP");
+    return _tempnam(dir ? dir : ".", "otftmp.");
+# endif
+}
+#endif
+
 int
 temporary_file(String &filename, ErrorHandler *errh)
 {
     if (no_create)
 	return 0;		// random number suffices
 
-#ifdef HAVE_MKSTEMP
+#if HAVE_MKSTEMP
     const char *tmpdir = getenv("TMPDIR");
     if (tmpdir)
 	filename = String(tmpdir) + "/otftotfm.XXXXXX";
@@ -166,7 +198,7 @@ temporary_file(String &filename, ErrorHandler *errh)
     return fd;
 #else  // !HAVE_MKSTEMP
     for (int tries = 0; tries < 5; tries++) {
-	if (!(filename = tmpnam(0)))
+	if (!(filename = my_tmpnam()))
 	    return errh->error("cannot create temporary file");
 # ifdef O_EXCL
 	int fd = ::open(filename.c_str(), O_RDWR | O_CREAT | O_EXCL | O_TRUNC, 0600);
